import numpy as np
import cv2
from trackingFunctions import calculateAngle
from trackingFunctions import distBetweenThetas
from trackingFunctions import assignValueIfBetweenRange
import math
from scipy.interpolate import UnivariateSpline
from numpy import linspace
import os.path
import csv
from scipy.interpolate import interp1d

def smoothTail(points, nbTailPoints):

  y = points[0]
  x = linspace(0, 1, len(y))
    
  if len(x) > 3:
    
    points2 = []
    for i in range(0, len(points[0])-1):
      if not((points[0][i] == points[0][i+1]) and (points[1][i] == points[1][i+1])):
        points2.append([points[0][i], points[1][i]])
    
    i = len(points[0]) - 1
    if not((points[0][i-1] == points[0][i]) and (points[1][i-1] == points[1][i])):
      points2.append([points[0][i], points[1][i]])
    
    points = np.array(points2).T
    
    # Define some points:
    points = np.array([points[0], points[1]]).T  # a (nbre_points x nbre_dim) array

    # Linear length along the line:
    distance = np.cumsum( np.sqrt(np.sum( np.diff(points, axis=0)**2, axis=1 )) )
    distance = np.insert(distance, 0, 0)/distance[-1]

    # Interpolation for different methods:
    interpolation_method = 'quadratic'    # 'slinear', 'quadratic', 'cubic'
    alpha = np.linspace(0, 1, nbTailPoints)

    interpolated_points = {}
    
    interpolator =  interp1d(distance, points, kind=interpolation_method, axis=0)
    interpolated_points = interpolator(alpha)

    interpolated_points = interpolated_points.T
    
    newX = interpolated_points[0]
    newY = interpolated_points[1]
    
  else:
 
    newX = points[0]
    newY = points[1]
    
  return [newX, newY]

def appendPoint(x, y, points):
  curPoint = np.zeros((2, 1))
  curPoint[0] = x;
  curPoint[1] = y;
  points = np.append(points, curPoint, axis=1)
  return points

def findNextPoints(depth,x,y,frame,points,angle,maxDepth,steps,nbList,initialImage,debug):
  
  lenX = len(frame[0]) - 1
  lenY = len(frame) - 1

  thetaDiffAccept = 1 #0.4
  
  if depth < 0.15*maxDepth:
    thetaDiffAccept = 0.4
  
  if depth > 0.85*maxDepth:
    thetaDiffAccept = 0.6
  
  pixTotMax = 1000000
  maxTheta  = angle

  l = [i*(math.pi/nbList) for i in range(0,2*nbList) if distBetweenThetas(i*(math.pi/nbList), angle) < thetaDiffAccept]
  
  if debug:
    print("debug")
  
  xTot = assignValueIfBetweenRange(x + steps[0], 0, lenX)
  yTot = assignValueIfBetweenRange(y, 0, lenY)
  if yTot == 0:
    yTot = 400
  
  for step in steps:
  
    for theta in l:
      
      xNew = assignValueIfBetweenRange(int(x + step * (math.cos(theta))), 0, lenX)
      yNew = assignValueIfBetweenRange(int(y + step * (math.sin(theta))), 0, lenY)
      pixTot = frame[yNew][xNew]
      
      if debug:
        print([theta,pixTot])
      
      # Keeps that theta angle as maximum if appropriate
      if (pixTot < pixTotMax):
        pixTotMax = pixTot
        maxTheta = theta
        if depth < 0.4*maxDepth:
          if xNew > x:
            xTot = xNew
            yTot = yNew
        else:
          xTot = xNew
          yTot = yNew
  
  w = 8 # THIS IS IMPORTANT
  ym = yTot - w
  yM = yTot + w
  xm = xTot - w
  xM = xTot + w
  if ym < 0:
    ym = 0
  if xm < 0:
    xm = 0
  if yM > len(initialImage):
    yM = len(initialImage)
  if xM > len(initialImage[0]):
    xM = len(initialImage[0])
  
  pixSur = np.min(initialImage[ym:yM, xm:xM])
  if debug:
    print("depth:", depth, " ; maxDepth:", maxDepth, " ; pixSur:", pixSur)
  
  # if depth > 0.95*maxDepth:
    # pixTot = frame[y][x]
    # if (pixTot < pixTotMax):
      # pixTotMax = pixTot
      # maxTheta = theta
      # xTot = x
      # yTot = y
      # depth = maxDepth + 10
  
  if debug:
    print(["max:",maxTheta,pixTotMax])
  
  # Calculates distance between new and old point
  distSubsquentPoints = math.sqrt((xTot - x)**2 + (yTot - y)**2)
  
  pixSurMax = 150
  if ((pixSur < pixSurMax) or (depth < 2*0.85*maxDepth)):
    points = appendPoint(xTot, yTot, points)
    if debug:
      cv2.circle(frame, (xTot, yTot), 3, (255,0,0),   -1)
      cv2.imshow('HeadEmbeddedTailTracking', frame)
      cv2.waitKey(0)
    
  newTheta = calculateAngle(x,y,xTot,yTot)
  
  if (distSubsquentPoints > 0) and (depth < 2*maxDepth) and (xTot < 1280 - 10) and (yTot > 10) and (yTot < 1024 - 10) and ((pixSur < pixSurMax) or (depth < 2*0.85*maxDepth)):
    (points,nop) = findNextPoints(depth+distSubsquentPoints,xTot,yTot,frame,points,newTheta,maxDepth,steps,nbList,initialImage,debug)
  
  return (points,newTheta)


def headEmbededTailTrackingTeresaNicolson(headPosition,nbTailPoints,i,thresh1,frame,hyperparameters,heading,maxDepth,tailTip,threshForBlackFrames):

  steps   = hyperparameters["step"]
  nbList  = 10
  
  x = headPosition[0]
  y = headPosition[1]
  
  initialImage = frame.copy()
  
  gaussian_blur = hyperparameters["headEmbededParamGaussianBlur"]
  frame = cv2.GaussianBlur(frame, (gaussian_blur, gaussian_blur), 0)
  # angle = hyperparameters["headEmbededParamInitialAngle"]
  angle = calculateAngle(x, y, tailTip[0], tailTip[1])
  
  points = np.zeros((2, 0))
  
  if np.mean(np.mean(frame)) > threshForBlackFrames:
    (points, lastFirstTheta2) = findNextPoints(0,x,y,frame,points,angle,maxDepth,steps,nbList,initialImage,hyperparameters["debugHeadEmbededFindNextPoints"])
    points = np.insert(points, 0, headPosition, axis=1) 
    if len(points[0]) > 3:
      points = smoothTail(points, nbTailPoints)
    points[0][0] = headPosition[0]
    points[1][0] = headPosition[1]
  else:
    points = np.zeros((2, nbTailPoints))
  
  output = np.zeros((1, len(points[0]), 2))

  for idx, x in enumerate(points[0]):
    output[0][idx][0] = x
    output[0][idx][1] = points[1][idx]

  return output


def headEmbededTailTrackFindMaxDepthTeresaNicolson(headPosition,nbTailPoints,i,x,y,thresh1,frame,hyperparameters,oppHeading,tailTip):

  x = headPosition[0]
  y = headPosition[1]

  steps   = hyperparameters["step"]
  nbList  = 10
  
  initialImage = frame.copy()
  
  gaussian_blur = hyperparameters["headEmbededParamGaussianBlur"]
  frame = cv2.GaussianBlur(frame, (gaussian_blur, gaussian_blur), 0)
  
  angle = calculateAngle(x, y, tailTip[0], tailTip[1])
  
  points = np.zeros((2, 0))
  
  (points, lastFirstTheta2) = findNextPoints(0,x,y,frame,points,angle,hyperparameters["headEmbededTailTrackFindMaxDepthInitialMaxDepth"],steps,nbList,initialImage, hyperparameters["debugHeadEmbededFindNextPoints"])
  
  distToTip        = np.full((200),10000)
  curTailLengthTab = np.full((200),10000)
  curTailLength  = 0
  k = 0
  
  distFromHeadToTip = abs(math.sqrt((x-tailTip[0])**2 + (y-tailTip[1])**2))
  while (curTailLength < 1.5*distFromHeadToTip) and (k < len(points[0])-1):
    curTailLength = curTailLength + abs(math.sqrt((points[0,k]-points[0,k+1])**2 + (points[1,k]-points[1,k+1])**2))
    distToTip[k]  = abs(math.sqrt((points[0,k]-tailTip[0])**2 + (points[1,k]-tailTip[1])**2))
    curTailLengthTab[k] = curTailLength
    k = k + 1
  
  minDistToTip    = 1000000
  indMinDistToTip = 0
  for idx, dist in enumerate(distToTip):
    if dist < minDistToTip:
      minDistToTip = dist
      indMinDistToTip = idx
  
  return (curTailLengthTab[indMinDistToTip] )
  
  
def getMeanOfImageOverVideo(videoPath):
  cap = cv2.VideoCapture(videoPath)
  meanss = []
  ret = True
  i = 0
  while (i < 100):
    ret, frame = cap.read()
    if ret:
      val = np.mean(np.mean(frame))
      meanss.append(val)
    i = i +1
  return np.mean(meanss)
  



import cv2
import numpy as np

from getForegroundImageSequential import getForegroundImageSequential
from getForegroundImage import getForegroundImage
from headEmbededFrame import headEmbededFrame
from headEmbededFrameSequential import headEmbededFrameSequential
from headEmbededFrameSequentialBackExtract import headEmbededFrameSequentialBackExtract
from headEmbededFrameBackExtract import headEmbededFrameBackExtract

def getImages(hyperparameters, cap, videoPath, i, background, wellNumber, wellPositions):
  if hyperparameters["headEmbeded"]:
    if hyperparameters["headEmbededRemoveBack"] == 0:
      if hyperparameters["adjustHeadEmbededTracking"] == 0:
        [frame, thresh1] = headEmbededFrameSequential(cap, videoPath, i)
      else:
        [frame, thresh1] = headEmbededFrame(videoPath, i)
      gray = frame.copy()
    else:
      if hyperparameters["adjustHeadEmbededTracking"] == 0:
        [frame, thresh1] = headEmbededFrameSequentialBackExtract(cap, videoPath, background, hyperparameters, i)
      else:
        [frame, thresh1] = headEmbededFrameBackExtract(videoPath, background, hyperparameters, i)
      gray = frame.copy()
  else:
    if hyperparameters["adjustFreelySwimTracking"] == 0:
      frame = getForegroundImageSequential(cap, videoPath, background, i, wellNumber, wellPositions, hyperparameters)
    else:
      frame = getForegroundImage(videoPath, background, i, wellNumber, wellPositions, hyperparameters)
    gray = frame.copy()
    
    ret,thresh1 = cv2.threshold(gray,hyperparameters["thresholdForBlobImg"],255,cv2.THRESH_BINARY)
  
  if hyperparameters["invertBlackWhiteOnImages"]:
    frame   = 255 - frame
    gray    = 255 - gray
    thresh1 = 255 - thresh1
  
  
  coverHorizontalPortionBelowForHeadDetect = hyperparameters["coverHorizontalPortionBelowForHeadDetect"]
  if coverHorizontalPortionBelowForHeadDetect != -1:
    gray[coverHorizontalPortionBelowForHeadDetect:len(gray)-1,:] = 255
    
  coverHorizontalPortionAboveForHeadDetect = hyperparameters["coverHorizontalPortionAboveForHeadDetect"]
  if coverHorizontalPortionAboveForHeadDetect != -1:
    gray[0:coverHorizontalPortionAboveForHeadDetect,:] = 255
    
  coverVerticalPortionRightForHeadDetect = hyperparameters["coverVerticalPortionRightForHeadDetect"]
  if coverVerticalPortionRightForHeadDetect != -1:
    gray[:,coverVerticalPortionRightForHeadDetect:len(gray[0])-1] = 255
  
  
  coverPortionForHeadDetect = hyperparameters["coverPortionForHeadDetect"]
  if coverPortionForHeadDetect == "Bottom":
    gray[int(len(gray)/2):len(gray)-1, :] = 255
  if coverPortionForHeadDetect == "Top":
    gray[0:int(len(gray)/2), :] = 255
  if coverPortionForHeadDetect == "Left":
    gray[:, 0:int(len(gray[0])/2)] = 255
  if coverPortionForHeadDetect == "Right":
    gray[:, int(len(gray[0])/2):len(gray[0])-1] = 255
  
  if hyperparameters["debugCoverHorizontalPortionBelow"] and i==firstFrame:
    cv2.imshow('Frame', gray)
    cv2.waitKey(0)
    cv2.destroyWindow('Frame')
    
  blur = cv2.GaussianBlur(gray, (hyperparameters["paramGaussianBlur"], hyperparameters["paramGaussianBlur"]),0)
  frame2 = frame.copy()
  erodeSize = hyperparameters["erodeSize"]
  kernel  = np.ones((erodeSize,erodeSize), np.uint8)
  
  thresh1 = cv2.erode(thresh1, kernel, iterations=hyperparameters["erodeIter"])
  thresh2 = thresh1.copy()
  thresh2 = cv2.dilate(thresh2, kernel, iterations=hyperparameters["dilateIter"])
  
  return [frame, gray, thresh1, blur, thresh2, frame2]



import numpy as np
import cv2
import cvui
from trackingFunctions import calculateAngle
from trackingFunctions import distBetweenThetas
from trackingFunctions import assignValueIfBetweenRange
import math
from scipy.interpolate import UnivariateSpline
from numpy import linspace
import os.path
import csv
  
# ix,  iy  = -1,-1
# ix2, iy2 = -1,-1

# mouse callback function
# def getXYCoordinates(event,x,y,flags,param):
  # global ix,iy
  # if event == cv2.EVENT_LBUTTONDOWN:
    # ix,iy = x,y
    
# def getXYCoordinates2(event,x2,y2,flags,param):
  # global ix2,iy2
  # if event == cv2.EVENT_LBUTTONDOWN:
    # ix2,iy2 = x2,y2

def findTailTipByUserInput(frame,hyperparameters):
  # global ix 
  # ix = -1
  # img = np.zeros((512,512,3), np.uint8)
  # cv2.namedWindow('Click on tail tip')
  # cv2.setMouseCallback('Click on tail tip',getXYCoordinates)
  # print("ix:", ix)
  # while(ix == -1):
    # cv2.imshow('Click on tail tip',frame)
    # k = cv2.waitKey(20) & 0xFF
    # if k == 27:
      # break
    # elif k == ord('a'):
      # print("yeah:",ix,iy)
  # cv2.destroyAllWindows()
  # return [ix,iy]
  
  WINDOW_NAME = "Click on tail tip"
  cvui.init(WINDOW_NAME)
  cv2.moveWindow(WINDOW_NAME, 0, 0)
  cvui.imshow(WINDOW_NAME, frame)
  while not(cvui.mouse(WINDOW_NAME, cvui.CLICK)):
    cursor = cvui.mouse(WINDOW_NAME)
    if cv2.waitKey(20) == 27:
      break
  # cv2.destroyAllWindows()
  cv2.destroyWindow(WINDOW_NAME)
  return [cursor.x, cursor.y]
  
def findHeadPositionByUserInput(frame):
  # img = np.zeros((512,512,3), np.uint8)
  # cv2.namedWindow('Click on the base of the tail')
  # cv2.setMouseCallback('Click on the base of the tail',getXYCoordinates2)
  # while(ix2 == -1):
    # cv2.imshow('Click on the base of the tail',frame)
    # k = cv2.waitKey(20) & 0xFF
    # if k == 27:
      # break
    # elif k == ord('a'):
      # print("yeah:",ix2,iy2)
  # cv2.destroyAllWindows()
  # return [ix2,iy2]
  
  WINDOW_NAME = "Click on the base of the tail"
  cvui.init(WINDOW_NAME)
  cv2.moveWindow(WINDOW_NAME, 0,0)
  cvui.imshow(WINDOW_NAME, frame)
  while not(cvui.mouse(WINDOW_NAME, cvui.CLICK)):
    cursor = cvui.mouse(WINDOW_NAME)
    if cv2.waitKey(20) == 27:
      break
  # cv2.destroyAllWindows()
  cv2.destroyWindow(WINDOW_NAME)
  return [cursor.x, cursor.y]

def getTailTipByFileSaved(hyperparameters,videoPath):
  ix = -1
  iy = -1
  with open(videoPath+'.csv') as csv_file:
    csv_reader = csv.reader(csv_file, delimiter=',')
    line_count = 0
    for row in csv_reader:
      if len(row):
        ix = row[0]
        iy = row[1]
  return [int(ix),int(iy)]

def getHeadPositionByFileSaved(videoPath):
  ix = -1
  iy = -1
  with open(videoPath+'HP.csv') as csv_file:
    csv_reader = csv.reader(csv_file, delimiter=',')
    line_count = 0
    for row in csv_reader:
      if len(row):
        ix = row[0]
        iy = row[1]
  return [int(ix),int(iy)]
  


import setuptools
from distutils.core import setup
setup(
  name = 'zebrazoom',
  version = '0.20',
  license='AGPL-3.0',
  description = 'Track and analyze zebrafish and animal behavior',
  author = 'Olivier Mirat',
  author_email = 'olivier.mirat.om@gmail.com',
  url = 'https://github.com/oliviermirat/ZebraZoom',
  download_url = 'https://github.com/oliviermirat/ZebraZoom/archive/v1.0.tar.gz',
  keywords = ['Animal', 'Behavior', 'Tracking', 'Zebrafish'],
  install_requires=[
    "scikit-learn",
    "h5py",
    "numpy",
    "matplotlib",
    "scipy",
    "pandas",
    "filterpy",
    "cvui",
    "opencv-python>=4.2.0.32",
    "xlrd"
  ],
  packages=setuptools.find_packages(),
  data_files=[
    (
      "zebrazoom",
      [
        "zebrazoom/configuration/4wellsZebrafishLarvaeEscapeResponses.json",
        "zebrazoom/configuration/fliesInTube.json",
        "zebrazoom/configuration/headEmbeddedZebrafishLarva.json",
        "zebrazoom/ZZoutput/example1/results_example1.txt",
        "zebrazoom/ZZoutput/example2/results_example2.txt",
        "zebrazoom/ZZoutput/example3/results_example3.txt",
        "zebrazoom/code/defaultConfigFile.json",
        "zebrazoom/code/GUI/blobCenter.png",
        "zebrazoom/code/GUI/blobExtremity.png",
        "zebrazoom/code/GUI/leftborder.png",
        "zebrazoom/code/GUI/no1.png",
        "zebrazoom/code/GUI/no2.png",
        "zebrazoom/code/GUI/ok1.png",
        "zebrazoom/code/GUI/rightborder.png",
        "zebrazoom/code/GUI/savedjustincase.png",
        "zebrazoom/dataAnalysis/experimentOrganizationExcel/example.xls"
      ],
    )
  ],
  include_package_data=True,
  classifiers=[
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
  ],
)


import cv2
import math
import numpy as np

from trackingFunctions import calculateAngle
from trackingFunctions import distBetweenThetas

def computeHeading(thresh1, x, y, headSize, hyperparameters):
  
  videoWidth  = hyperparameters["videoWidth"]
  videoHeight = hyperparameters["videoHeight"]
  
  if y-headSize < 0:
    ymin = 0
  else:
    ymin = y-headSize
    
  if y+headSize > videoHeight:
    ymax = videoHeight - 1
  else:
    ymax = y+headSize
    
  if x-headSize < 0:
    xmin = 0
  else:
    xmin = x-headSize
    
  if x+headSize > videoWidth:
    xmax = videoWidth - 1
  else:
    xmax = x+headSize
  
  img = thresh1[int(ymin):int(ymax), int(xmin):int(xmax)]
  
  img[0,:] = 255
  
  img[len(img)-1,:] = 255
  img[:,0] = 255
  img[:,len(img[0])-1] = 255
  
  if hyperparameters["debugHeadingCalculation"]:
    cv2.imshow('imgForHeadingCalculation', img)
    cv2.waitKey(0)
  
  y, x = np.nonzero(img)
  x = x - np.mean(x)
  y = y - np.mean(y)
  coords = np.vstack([x, y])
  cov = np.cov(coords)
  evals, evecs = np.linalg.eig(cov)
  sort_indices = np.argsort(evals)[::-1]
  x_v1, y_v1 = evecs[:, sort_indices[0]]  # Eigenvector with largest eigenvalue
  x_v2, y_v2 = evecs[:, sort_indices[1]]
  scale = 20
  
  theta = calculateAngle(0, 0, x_v1, y_v1)
  theta = (theta - math.pi/2) % (2 * math.pi)
  
  return theta


def calculateHeading(x, y, i, thresh1, thresh2, takeTheHeadClosestToTheCenter, hyperparameters):

  nbList = hyperparameters["nbList"]
  expDecreaseFactor = hyperparameters["expDecreaseFactor"]
  firstFrame = hyperparameters["firstFrame"]
  lastFrame = hyperparameters["lastFrame"]
  minAreaBody = hyperparameters["minAreaBody"]
  maxAreaBody = hyperparameters["maxAreaBody"]
  headSize = hyperparameters["headSize"]
  debugTracking = hyperparameters["debugTracking"]
  headEmbeded = hyperparameters["headEmbeded"]
  
  if hyperparameters["debugHeadingCalculation"]:
    cv2.imshow('thresh1', thresh1)
    cv2.waitKey(0)
    cv2.imshow('thresh2', thresh2)
    cv2.waitKey(0)
  
  cx = 0
  cy = 0
  cxNew = 0
  cyNew = 0
  
  minAreaCur = minAreaBody
  maxAreaCur = maxAreaBody
  while (cx == 0) and (minAreaCur > -200):
    
    thresh1[0,:]                 = 255
    thresh1[len(thresh1)-1,:]    = 255
    thresh1[:,0]                 = 255
    thresh1[:,len(thresh1[0])-1] = 255
    
    contours, hierarchy = cv2.findContours(thresh1,cv2.RETR_TREE,cv2.CHAIN_APPROX_SIMPLE)
    for contour in contours:
      area = cv2.contourArea(contour)
      if (area > minAreaCur) and (area < maxAreaCur):
        M = cv2.moments(contour)
        if takeTheHeadClosestToTheCenter == 0:
          if M['m00']:
            cx = int(M['m10']/M['m00'])
            cy = int(M['m01']/M['m00'])
          else:
            cx = 0
            cy = 0
        else:
          if M['m00']:
            cxNew = int(M['m10']/M['m00'])
            cyNew = int(M['m01']/M['m00'])
          else:
            cx = 0
            cy = 0
          
          distToCenterNew = math.sqrt((cxNew-x)**2 + (cyNew-y)**2)
          distToCenter    = math.sqrt((cx-x)**2    + (cy-y)**2)
          # print("distToCenter:",distToCenter," ; distToCenterNew:",distToCenterNew)
          if distToCenterNew < distToCenter:
            cx = cxNew
            cy = cyNew
            # print("change realized, new center is:", cx, cy)
    minAreaCur = minAreaCur - 100
    maxAreaCur = maxAreaCur + 100
  
  
  heading = computeHeading(thresh2, x, y, headSize, hyperparameters)
  lastFirstTheta = calculateAngle(x, y, cx, cy)
  
  headingApproximate = lastFirstTheta
  headingPreciseOpt1 = heading
  headingPreciseOpt2 = (heading + math.pi) % (2*math.pi)
  diffAngle1 = distBetweenThetas(headingApproximate, headingPreciseOpt1)
  diffAngle2 = distBetweenThetas(headingApproximate, headingPreciseOpt2)
  if (diffAngle1 > diffAngle2):
    heading = headingPreciseOpt1
  else:
    heading = headingPreciseOpt2
  
  return [heading, lastFirstTheta]


def calculateHeadingSimple(x, y, thresh2, hyperparameters):
  
  headSize = hyperparameters["headSize"]
  heading = computeHeading(thresh2, x, y, headSize, hyperparameters)
  
  return heading



import scipy.io
import pandas as pd
import json
import numpy as np
from getDynamicParameters import getDynamicParameters
from getTailAngles import getTailAngles
from getInstaSpeed import getInstaSpeed
from getInstaHeadingDiff import getInstaHeadingDiff
from getInstaHorizontalDisplacement import getInstaHorizontalDisplacement
from getGlobalParameters import getGlobalParameters
from getDeltaHead import getDeltaHead
from getTailLength  import getTailLength
from getTailLength2 import getTailLength2
from getTailAngleRecalculated import getTailAngleRecalculated
from getTailAngleRecalculated2 import getTailAngleRecalculated2
import pickle

def createDataFrame(dataframeOptions):

  pathToExcelFile                   = dataframeOptions['pathToExcelFile']
  nameOfFile                        = dataframeOptions['nameOfFile']
  fileExtension                     = dataframeOptions['fileExtension']
  smoothingFactor                   = dataframeOptions['smoothingFactorDynaParam']
  resFolder                         = dataframeOptions['resFolder']
  numberOfBendsIncludedForMaxDetect = dataframeOptions['numberOfBendsIncludedForMaxDetect']
  minNbBendForBoutDetect            = dataframeOptions['minNbBendForBoutDetect']
  computeTailAngleParamForCluster   = dataframeOptions['computeTailAngleParamForCluster']
  computeMassCenterParamForCluster  = dataframeOptions['computeMassCenterParamForCluster']
  if 'defaultZZoutputFolderPath' in dataframeOptions:
    defaultZZoutputFolderPath       = dataframeOptions['defaultZZoutputFolderPath']
  else:
    defaultZZoutputFolderPath       = ''
  computetailAnglesRecalculatedParamsForCluster = False
  if 'computetailAnglesRecalculatedParamsForCluster' in dataframeOptions:
    computetailAnglesRecalculatedParamsForCluster = dataframeOptions["computetailAnglesRecalculatedParamsForCluster"]
  else:
    computetailAnglesRecalculatedParamsForCluster = False
  
  nbFramesTakenIntoAccount          = dataframeOptions['nbFramesTakenIntoAccount']
  excelFile = pd.read_excel(pathToExcelFile + nameOfFile + fileExtension)
  if nbFramesTakenIntoAccount == -1:
    boutNbFrames = []
    boutTakenIntoAcccount = 0
    videoId = 0
    if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder":
      path    = defaultZZoutputFolderPath + excelFile.loc[videoId, 'trial_id'] + '/'
    else:
      path    = excelFile.loc[videoId, 'path'] + excelFile.loc[videoId, 'trial_id'] + '/'
    trial_id  = excelFile.loc[videoId, 'trial_id']
    include   = excelFile.loc[videoId, 'include']
    include = eval('[' + include + ']')
    include = include[0]
    with open(path+'results_'+trial_id+'.txt') as f:
      supstruct = json.load(f)
    for Well_ID, Cond in enumerate(include):
      if include[Well_ID]:
        for NumBout, dataForBout in enumerate(supstruct["wellPoissMouv"][Well_ID][0]):
          boutNbFrames.append(len(dataForBout["HeadX"]))
          boutTakenIntoAcccount = boutTakenIntoAcccount + 1
          if boutTakenIntoAcccount > 100:
            break;
      if boutTakenIntoAcccount > 100:
        break;
    nbFramesTakenIntoAccount = int(np.median(boutNbFrames))
  
  genotypes  = []
  conditions = []
  
  # Creating labels of columns of dataframe
  # Global parameters
  globParam  = ['Well_ID', 'NumBout', 'BoutStart', 'BoutEnd', 'Condition', 'Genotype', 'BoutDuration', 'TotalDistance', 'Speed', 'NumberOfOscillations', 'meanTBF', 'maxAmplitude', 'deltaHead', 'xstart', 'xend', 'xmean', 'firstBendTime', 'firstBendAmplitude']
  # Tail angle related parameters for clustering
  instaTBF   = ['instaTBF'  + str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
  instaAmp   = ['instaAmp'  + str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
  instaAsym  = ['instaAsym' + str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
  tailAngles = ['tailAngles'+ str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
  tailAnglesRecalculated = ['tailAnglesRecalculated'+ str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
  tailAnglesRecalculated2 = ['tailAnglesRecalculated2_'+ str(i) for i in range(1, 7*nbFramesTakenIntoAccount+1)]
  # Center of mass related parameters for clustering
  instaSpeed       = ['instaSpeed'       + str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
  instaHeadingDiff = ['instaHeadingDiff' + str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
  instaHorizDispl  = ['instaHorizDispl'  + str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
  # Assembling columns
  dfCols = ['Trial_ID'] + globParam + tailAngles
  if computeTailAngleParamForCluster:
    dfCols = dfCols + instaTBF + instaAmp + instaAsym
  if computeMassCenterParamForCluster:
    dfCols = dfCols + instaSpeed + instaHeadingDiff + instaHorizDispl
  if computetailAnglesRecalculatedParamsForCluster:
    dfCols = dfCols + ['tailLength', 'tailLengthFromRecalculatedAngles'] + tailAnglesRecalculated + tailAnglesRecalculated2
  if computeTailAngleParamForCluster or computeMassCenterParamForCluster or computetailAnglesRecalculatedParamsForCluster:
    dfCols = dfCols + ['classification']
  numberOfParameters = len(dfCols)
  
  # Creating an empty dataframe
  params = np.zeros((0, numberOfParameters))
  dfParam = pd.DataFrame(params,columns=dfCols)
  trialidstab = []
  
  # Filling in the dataframe
  # curBoutId = 0
  print("Calculating and storing all parameters:")
  for videoId in range(0, len(excelFile)):
    if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder":
      path    = defaultZZoutputFolderPath + excelFile.loc[videoId, 'trial_id'] + '/'
    else:
      path    = excelFile.loc[videoId, 'path'] + excelFile.loc[videoId, 'trial_id'] + '/'
    trial_id  = excelFile.loc[videoId, 'trial_id']
    fq        = excelFile.loc[videoId, 'fq']
    pixelsize = excelFile.loc[videoId, 'pixelsize']
    condition = excelFile.loc[videoId, 'condition']
    condition = eval('[' + condition + ']')
    condition = condition[0]
    genotype  = excelFile.loc[videoId, 'genotype']
    genotype = eval('[' + genotype + ']')
    genotype = genotype[0]
    include   = excelFile.loc[videoId, 'include']
    include = eval('[' + include + ']')
    include = include[0]
    
    with open(path+'results_'+trial_id+'.txt') as f:
      supstruct = json.load(f)
    
    for Well_ID, Cond in enumerate(condition):
      if include[Well_ID]:
        print("trial_id:", trial_id, " ; Well_ID:", Well_ID)
        dfParamSub = pd.DataFrame(params,columns=dfCols)
        curBoutId = 0
        for NumBout, dataForBout in enumerate(supstruct["wellPoissMouv"][Well_ID][0]):
          if type(dataForBout["Bend_Timing"]) == list and len(dataForBout["Bend_Timing"]) >= minNbBendForBoutDetect and (not("flag" in dataForBout) or dataForBout["flag"] == 0):
          
            trialidstab.append(trial_id)
            if not(genotype[Well_ID] in genotypes):
              genotypes.append(genotype[Well_ID])
            if not(condition[Well_ID] in conditions):
              conditions.append(condition[Well_ID])
            
            # Calculating the global kinematic parameters and more and stores them the dataframe
            
            [BoutDuration, TotalDistance, Speed, NumberOfOscillations, meanTBF, maxAmplitude, xstart, xend, xmean, firstBendTime, firstBendAmplitude] = getGlobalParameters(dataForBout, fq, pixelsize)
            
            deltahead  = abs(getDeltaHead(dataForBout))
            tailLength = getTailLength(dataForBout)
            
            tailAnglesRecalculatedData  = getTailAngleRecalculated(dataForBout, nbFramesTakenIntoAccount, numberOfBendsIncludedForMaxDetect)
            
            # tailLengthFromRecalculatedAngles = getTailLength2(tailAnglesRecalculatedData)
            
            toPutInDataFrameColumn = []
            toPutInDataFrame       = []
            
            toPutInDataFrameColumn = toPutInDataFrameColumn + globParam
            toPutInDataFrame       = toPutInDataFrame + [Well_ID, NumBout, dataForBout['BoutStart'], dataForBout['BoutEnd'], condition[Well_ID], genotype[Well_ID], BoutDuration, TotalDistance, Speed, NumberOfOscillations, meanTBF, maxAmplitude, deltahead, xstart, xend, xmean, firstBendTime, firstBendAmplitude]
            
            toPutInDataFrameColumn = toPutInDataFrameColumn + tailAngles
            toPutInDataFrame       = toPutInDataFrame + getTailAngles(dataForBout, smoothingFactor, nbFramesTakenIntoAccount, numberOfBendsIncludedForMaxDetect)
            
            
            # Calculate "dynamic" tail angle related parameters for clustering
            
            if computeTailAngleParamForCluster:
            
              toPutInDataFrameColumn = toPutInDataFrameColumn + instaTBF + instaAmp + instaAsym
              toPutInDataFrame       = toPutInDataFrame + getDynamicParameters(dataForBout, smoothingFactor, nbFramesTakenIntoAccount, numberOfBendsIncludedForMaxDetect)
            
            # Calculate "dynamic" center of mass related parameters for clustering
            
            if computeMassCenterParamForCluster:
            
              instaSpeedVal       = getInstaSpeed(dataForBout, nbFramesTakenIntoAccount)
              instaHeadingDiffVal = getInstaHeadingDiff(dataForBout, nbFramesTakenIntoAccount)
              instaHorizDisplVal  = getInstaHorizontalDisplacement(dataForBout, nbFramesTakenIntoAccount)
              
              toPutInDataFrameColumn = toPutInDataFrameColumn + instaSpeed + instaHeadingDiff + instaHorizDispl
              toPutInDataFrame       = toPutInDataFrame + instaSpeedVal + instaHeadingDiffVal + instaHorizDisplVal
              
            # Recalculates tail angles and calculates 
            
            if computetailAnglesRecalculatedParamsForCluster:

              tailLength = getTailLength(dataForBout)
              
              tailAnglesRecalculatedData  = getTailAngleRecalculated(dataForBout, nbFramesTakenIntoAccount, numberOfBendsIncludedForMaxDetect)
              
              tailLengthFromRecalculatedAngles = getTailLength2(tailAnglesRecalculatedData)
            
              tailAnglesRecalculatedData2 = getTailAngleRecalculated2(dataForBout, nbFramesTakenIntoAccount, numberOfBendsIncludedForMaxDetect)
              
              toPutInDataFrameColumn = toPutInDataFrameColumn + ['tailLength', 'tailLengthFromRecalculatedAngles'] + tailAnglesRecalculated + tailAnglesRecalculated2
              toPutInDataFrame       = toPutInDataFrame + [tailLength, tailLengthFromRecalculatedAngles] + tailAnglesRecalculatedData + tailAnglesRecalculatedData2.tolist()
            
            dfParamSub.loc[curBoutId, toPutInDataFrameColumn] = toPutInDataFrame
            
            curBoutId = curBoutId + 1
        
        dfParam = pd.concat([dfParam, dfParamSub])
  
  # Saving the dataframe
  dfParam = dfParam.reset_index()
  dfParam['Trial_ID'][:] = trialidstab
  outfile = open(resFolder + nameOfFile,'wb')
  pickle.dump(dfParam,outfile)
  outfile.close()
  
  return [conditions, genotypes, nbFramesTakenIntoAccount]
